#!/bin/sh

help ()
{
    echo "EXEC_DIR is not defined."
    echo "Define EXEC_DIR to point to your target filesystem."
    exit 1
}

test -z $EXEC_DIR && help

useprefix=/usr
do_ssl_patch () 
{
    tar zxf openssl-0.9.8m.tar.gz
    cd openssl-0.9.8m
    patch -p1  < ../wpa_supp_patch/configure-targets.patch
    patch -p1  < ../wpa_supp_patch/shared-libs.patch
    patch -p1  < ../wpa_supp_patch/debian.patch
    patch -p1  < ../wpa_supp_patch/oe-ldflags.patch
}

do_ssl_configure() 
{
    cd util
    perl perlpath.pl /usr/bin
    cd ..
    ln -sf apps/openssl.pod crypto/crypto.pod ssl/ssl.pod doc/

    target=linux-elf-arm
    # inject machine-specific flags
    sed -i -e "s|^\(\"$target\",\s*\"[^:]\+\):\([^:]\+\)|\1:-DL_ENDIAN      -DTERMIO -fexpensive-optimizations -frename-registers -fomit-frame-pointer -O2 -ggdb3 -Wall|g" Configure
    perl ./Configure shared --prefix=$useprefix --openssldir=$useprefix/usr/lib/ssl $target
}

do_ssl_compile () 
{
    make
}

do_ssl_install_host () 
{
    # copy the files on linux-devkit directory
    EXEC_HOST_DIR=${DVSDK}/linux-devkit/arm-none-linux-gnueabi
    make INSTALL_PREFIX="${EXEC_HOST_DIR}" install
    mkdir -p ${EXEC_HOST_DIR}/usr/lib 
    cp libcrypto* ${EXEC_HOST_DIR}/usr/lib
    cp libssl* ${EXEC_HOST_DIR}/usr/lib
    mkdir -p ${EXEC_HOST_DIR}/usr/include
    cp --dereference -R include/openssl ${EXEC_HOST_DIR}/usr/include
}

do_ssl_install_target () 
{
    export EXEC_DIR=$EXEC_DIR

    # copy on target filesystem
    sudo mkdir -p ${EXEC_DIR}/usr/lib
    sudo mkdir -p ${EXEC_DIR}/usr/bin
    sudo cp -dpR libcrypto.so* ${EXEC_DIR}/usr/lib/
    sudo cp -dpR libssl.so* ${EXEC_DIR}/usr/lib/
    sudo cp apps/openssl ${EXEC_DIR}/usr/bin/
    sudo mkdir -p ${EXEC_DIR}/usr/lib/ssl
    sudo cp apps/openssl.cnf ${EXEC_DIR}/usr/lib/ssl/
}

do_wpa_patch ()
{
    tar zxf wpa_supplicant-0.5.7.tar.gz
    cd wpa_supplicant-0.5.7
    patch -p1 < ../wpa_supp_patch/wpa-0.5.7-libc-types.patch
    patch -p1 < ../wpa_supp_patch/wpa_suppl.diff
    cp ../wpa_supp_patch/defconfig-openssl .config
}

do_wpa_compile () 
{
    export LDFLAGS="-L${DVSDK}/linux-devkit/arm-none-linux-gnueabi/usr/lib -Wl,-rpath-link,${DVSDK}/linux-devkit/arm-none-linux-gnueabi/usr/lib -Wl,-O1 -Wl,--hash-style=gnu "

    make CROSS_COMPILE=arm-none-linux-gnueabi-
}

do_wpa_install_host ()
{
    install -d ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/etc/wpa_supplicant
    install -m 755 ../wpa_supp_patch/ifupdown.sh ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/etc/wpa_supplicant
    install -m 755 ../wpa_supp_patch/functions.sh ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/etc/wpa_supplicant
    install -d ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/usr
    install -d ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/usr/sbin
    install -m 755 wpa_supplicant ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/usr/sbin
    install -m 755 wpa_passphrase ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/usr/sbin
    install -m 755 wpa_cli ${DVSDK}/linux-devkit/arm-none-linux-gnueabi/usr/sbin
}

do_wpa_install_target ()
{
    # copy on target filesystem
    sudo install -m 755 wpa_supplicant ${EXEC_DIR}/usr/sbin
    sudo install -m 755 wpa_passphrase ${EXEC_DIR}/usr/sbin
    sudo install -m 755 wpa_cli ${EXEC_DIR}/usr/sbin
    sudo install -d ${EXEC_DIR}/var/run/wpa_supplicant
    sudo install -d ${EXEC_DIR}/etc/network/if-pre-up.d/
    sudo install -d ${EXEC_DIR}/etc/network/if-post-down.d/
    sudo install -d ${EXEC_DIR}/etc/network/if-down.d/
    sudo install -d ${EXEC_DIR}/etc/wpa_supplicant
    sudo install -m 755 ../wpa_supp_patch/ifupdown.sh ${EXEC_DIR}/etc/wpa_supplicant
    sudo install -m 755 ../wpa_supp_patch/functions.sh ${EXEC_DIR}/etc/wpa_supplicant
    sudo ln -s /etc/wpa_supplicant/ifupdown.sh ${EXEC_DIR}/etc/network/if-pre-up.d/wpasupplicant
    sudo ln -s /etc/wpa_supplicant/ifupdown.sh ${EXEC_DIR}/etc/network/if-post-down.d/wpasupplicant
}

do_ssl_patch
#cd openssl-0.9.8m
do_ssl_configure
do_ssl_compile 
do_ssl_install_host 
do_ssl_install_target 

cd ..
do_wpa_patch
#cd wpa_supplicant-0.5.7
do_wpa_compile
do_wpa_install_host
do_wpa_install_target
